use clap::{Parser, Subcommand};

#[derive(Parser, Debug)]
#[clap(author, version, about, long_about = None)]
pub struct Cli {
	/// Log level
	#[clap(short, long, default_value = "warn")]
	pub log: String,

	/// Subcommand
	#[clap(subcommand)]
	pub command: Command,
}

#[derive(Subcommand, Debug)]
#[clap(about, long_about = None)]
pub enum Command {
	/// Manage profiles
	Profile {
		#[clap(subcommand)]
		command: ProfileCommand,
	},
	/// Manage students
	Student {
		#[clap(subcommand)]
		command: StudentCommand,
	},
	/// Synchronize the profile manually
	Sync,
}

#[derive(Subcommand, Debug)]
#[clap(about, long_about = None)]
pub enum ProfileCommand {
	/// List all profiles
	List,
	/// Select a profile by name
	Select { name: String },
	/// Register a new profile
	Register {
		/// Profile name
		name: String,
		/// Token from the "Dostęp mobilny" page
		token: String,
		/// Symbol from the "Dostęp mobilny" page
		symbol: String,
		/// PIN from the "Dostęp mobilny" page
		pin: String,
		/// Device name
		#[clap(default_value = "uonet-cli")]
		device_name: String,
		/// Don't select the profile after creating
		#[clap(short, long, action)]
		noset: bool,
	},
	/// Delete a profile by name
	///
	/// Note: This does not unregister the device from UONET+. You need to do that manually.
	Delete {
		/// Profile name
		name: String,
	},
}

#[derive(Subcommand, Debug)]
#[clap(about, long_about = None)]
pub enum StudentCommand {
	/// List students
	List,
	/// Select a student
	Select {
		/// ID of the student
		id: i32,
	},
}
