use std::time::SystemTime;

use uonet::account::VulcanAccount;
use colored::Colorize;

use crate::{storage::Profile, term};

pub async fn sync(profile: &mut Profile, force: bool) -> anyhow::Result<()> {
	let ts = SystemTime::now()
		.duration_since(SystemTime::UNIX_EPOCH)?
		.as_secs();
	if !force && profile.last_sync + 1800 > ts {
		return Ok(());
	}
	term::process("Starting synchronization");
	let mut account = VulcanAccount::from_credentials(profile.credentials.clone())?;
	term::process("Updating the students");
	let students = account
		.list_students()
		.await?;
	profile.students = students.clone();
	if let Some(old_student_id) = profile.student {
		if !students.iter().any(|v| v.pupil.id == old_student_id) {
			term::frog(format!("Student {} somehow vanished. Deselecting", old_student_id.to_string().red()).as_str());
			profile.student = None;
		}
	}
	
	for student in students {
		let displayname = format!("{} {}", student.pupil.first_name, student.pupil.surname);
		term::header(format!("Synchronizing student {}:", displayname.green()).as_str());
		account.set_student(student)?;
		term::process("Synchronizing grades");
		let grades = account.grades(None).await?;
		println!("{:?}", grades);
		term::done(format!("Student '{}' synchronized successfully", displayname.yellow()).as_str());
	}

	term::process("Saving changes to the profile");
	profile.last_sync = ts;
	profile.commit()?;

	term::done("Synchronization complete");
	Ok(())
}
