use crate::models::UonetStatus;
use openssl::error::ErrorStack;
use thiserror::Error;

/// Global result type used in the library
pub type Result<T> = std::result::Result<T, UonetError>;

/// Global error type used in the library
#[derive(Error, Debug)]
pub enum UonetError {
	#[error("An OpenSSL call failed: {0}")]
	OpenSSL(#[from] ErrorStack),
	#[error("A HTTP error has occurred: {0}")]
	Http(#[from] reqwest::Error),
	#[error("A JSON error has occurred: {0}")]
	Json(#[from] serde_json::Error),
	#[error("Firebase didn't return a valid token: {0}")]
	InvalidFirebaseToken(String),
	#[error("Incomplete or corrupted API response [{0:?}]")]
	IncompleteResponse(Option<String>),
	#[error("The provided token is invalid")]
	InvalidToken,
	#[error("The provided Student is invalid")]
	InvalidStudent,
	#[error("Invalid API URL: {0}")]
	InvalidUrl(String),
	#[error("Invalid device name: {0:?}")]
	InvalidDeviceName(String),
	#[error("API responded with an error code: {0:?}")]
	ErrorResponse(UonetStatus),
	#[error("No student selected")]
	StudentMissing,
}
