use chrono::{NaiveDate, NaiveTime};
use serde::{Deserialize, Serialize};

/// UONET+ API status code
#[derive(Debug, Clone, Deserialize)]
#[serde(rename_all = "PascalCase")]
pub struct UonetStatus {
	pub code: u32,
	pub message: String,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "PascalCase")]
pub(crate) struct UonetResponse<T> {
	pub status: UonetStatus,
	pub envelope: Option<T>,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "PascalCase")]
pub(crate) struct RegisterResponse {
	pub login_id: i32,
	pub user_name: String,
	#[serde(rename = "RestURL")]
	pub rest_url: String,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Student {
	pub class_display: String,
	pub top_level_partition: String,
	pub partition: String,
	pub pupil: Pupil,
	pub unit: Unit,
	pub constituent_unit: School,
	pub periods: Vec<Period>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Pupil {
	pub id: i32,
	pub login_id: i32,
	pub first_name: String,
	pub surname: String,
	pub sex: bool, // TRANSPHOBIC API
	pub second_name: Option<String>,
	pub login_value: Option<String>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Unit {
	pub id: i32,
	pub symbol: String,
	pub name: String,
	pub short: String,
	pub display_name: String,
	#[serde(rename = "RestURL")]
	pub rest_url: String,
	pub address: Option<String>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct School {
	pub id: i32,
	pub name: String,
	pub short: String,
	pub address: Option<String>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Period {
	pub id: i32,
	pub level: i32,
	pub number: i32,
	pub current: bool,
	pub last: bool,
	pub start: VulcanDateTime,
	pub end: VulcanDateTime,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct VulcanDateTime {
	// I wanna meet the person who decided to store date and time like this
	pub timestamp: i64,
	pub date: NaiveDate,
	pub time: NaiveTime,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Grade {
	pub id: i32,
	pub key: String,
	pub pupil_id: i32,
	pub content_raw: String,
	pub content: String,
	pub date_created: VulcanDateTime,
	pub date_modify: VulcanDateTime,
	pub creator: Teacher,
	pub modifier: Teacher,
	pub column: GradeColumn,
	pub value: Option<f32>,
	pub comment: Option<String>,
	pub numerator: Option<f32>,
	pub denominator: Option<f32>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Teacher {
	pub id: i32,
	pub name: String,
	pub surname: String,
	pub display_name: String,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct GradeColumn {
	pub id: i32,
	pub key: String,
	pub period_id: i32,
	pub name: String,
	pub code: String,
	pub number: i32,
	pub weight: f32,
	pub subject: Subject,
	pub group: Option<String>,
	pub category: Option<GradeCategory>,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct Subject {
	pub id: i32,
	pub key: String,
	pub name: String,
	pub kod: String,
	pub position: i32,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
#[serde(rename_all = "PascalCase")]
pub struct GradeCategory {
	pub id: i32,
	pub name: String,
	pub code: String,
}
